Method: GR.axeslbl

Defined in:
lib/gr.rb

.axeslblObject

Note:

This method uses GRCommons::Fiddley::Function as a callback function. Please read the source code If you have to use it. There are some examples of the use of this function in the Plot class..

Create axes in the current workspace and supply a custom function for changing the behaviour of the tick labels.

Similar to gr_axes() but allows more fine-grained control over tick labels and text positioning by supplying callback functions. Within the callback function you can use normal GR text primitives for performing any manipulations on the label text. See gr_axes() for more details on drawing axes.

  • fpx/fpy

    • param x [Numeric] NDC of the label in X direction.

    • param y [Numeric] NDC of the label in Y direction.

    • param svalue [String] Internal string representation of the text drawn by GR at (x,y).

    • param value [Numeric] Floating point representation of the label drawn at (x,y).

Parameters:

  • x_tick (Numeric)

    The interval between minor tick marks on the X axis.

  • y_tick (Numeric)

    The interval between minor tick marks on the Y axis.

  • x_org (Numeric)

    The world coordinate of the origin (point of intersection) of the X axis.

  • y_org (Numeric)

    The world coordinate of the origin (point of intersection) of the Y axis.

  • major_x (Integer)

    Unitless integer value specifying the number of minor tick intervals between major tick marks on the X axis. Values of 0 or 1 imply no minor ticks. Negative values specify no labels will be drawn for the associated axis.

  • major_y (Integer)

    Unitless integer value specifying the number of minor tick intervals between major tick marks on the Y axis. Values of 0 or 1 imply no minor ticks. Negative values specify no labels will be drawn for the associated axis.

  • tick_size (Numeric)

    The length of minor tick marks specified in a normalized device coordinate unit. Major tick marks are twice as long as minor tick marks. A negative value reverses the tick marks on the axes from inward facing to outward facing (or vice versa).

  • fpx (Pointer)

    Function pointer to a function that returns a label for a given tick on the X axis. The callback function should have the following arguments.

  • fpy (Pointer)

    Exactly same as the fpx above, but for the the Y axis.



# File 'lib/gr.rb', line 1165