Method: GR.mathtex

Defined in:
lib/gr.rb

.mathtexObject

Generate a character string starting at the given location. Strings can be defined to create mathematical symbols and Greek letters using LaTeX syntax.

Parameters:

  • x (Numeric)

    X coordinate of the starting position of the text string

  • y (Numeric)

    Y coordinate of the starting position of the text string

  • string (String)

    The text string to be drawn



# File 'lib/gr.rb', line 1873