Method: GR.settextfontprec
- Defined in:
- lib/gr.rb
.settextfontprec ⇒ Object
Specify the text font and precision for subsequent text output primitives.
The appearance of a font depends on the text precision value specified. STRING, CHARACTER or STROKE precision allows for a greater or lesser realization of the text primitives, for efficiency. STRING is the default precision for GR and produces the highest quality output using either native font rendering or FreeType. OUTLINE uses the GR path rendering functions to draw individual glyphs and produces the highest quality output.
|
|
# File 'lib/gr.rb', line 598
|