Method: GR.axeslbl
- Defined in:
- lib/gr.rb
.axeslbl ⇒ Object
This method uses GRCommons::Fiddley::Function as a callback function. Please read the source code If you have to use it. There are some examples of the use of this function in the Plot class..
Create axes in the current workspace and supply a custom function for changing the behaviour of the tick labels.
Similar to gr_axes() but allows more fine-grained control over tick labels and text positioning by supplying callback functions. Within the callback function you can use normal GR text primitives for performing any manipulations on the label text. See gr_axes() for more details on drawing axes.
-
fpx/fpy
-
param x [Numeric] NDC of the label in X direction.
-
param y [Numeric] NDC of the label in Y direction.
-
param svalue [String] Internal string representation of the text drawn by GR at (x,y).
-
param value [Numeric] Floating point representation of the label drawn at (x,y).
-
|
|
# File 'lib/gr.rb', line 1165
|