Method: GR.mathtex
- Defined in:
- lib/gr.rb
.mathtex ⇒ Object
Generate a character string starting at the given location. Strings can be defined to create mathematical symbols and Greek letters using LaTeX syntax.
|
|
# File 'lib/gr.rb', line 1873
|